@@ -157,16 +157,31 @@ int mosquittopp::connect(const char *host, int port, int keepalive)
 	return mosquitto_connect(m_mosq, host, port, keepalive);
 }
 
+int mosquittopp::connect(const char *host, int port, int keepalive, const char *bind_address)
+{
+	return mosquitto_connect_bind(m_mosq, host, port, keepalive, bind_address);
+}
+
 int mosquittopp::connect_async(const char *host, int port, int keepalive)
 {
 	return mosquitto_connect_async(m_mosq, host, port, keepalive);
 }
 
+int mosquittopp::connect_async(const char *host, int port, int keepalive, const char *bind_address)
+{
+	return mosquitto_connect_bind_async(m_mosq, host, port, keepalive, bind_address);
+}
+
 int mosquittopp::reconnect()
 {
 	return mosquitto_reconnect(m_mosq);
 }
 
+int mosquittopp::reconnect_async()
+{
+	return mosquitto_reconnect_async(m_mosq);
+}
+
 int mosquittopp::disconnect()
 {
 	return mosquitto_disconnect(m_mosq);
@@ -197,6 +212,16 @@ int mosquittopp::publish(int *mid, const char *topic, int payloadlen, const void
 	return mosquitto_publish(m_mosq, mid, topic, payloadlen, payload, qos, retain);
 }
 
+void mosquittopp::reconnect_delay_set(unsigned int reconnect_delay, unsigned int reconnect_delay_max, bool reconnect_exponential_backoff)
+{
+	mosquitto_reconnect_delay_set(m_mosq, reconnect_delay, reconnect_delay_max, reconnect_exponential_backoff);
+}
+
+int mosquittopp::max_inflight_messages_set(unsigned int max_inflight_messages)
+{
+	return mosquitto_max_inflight_messages_set(m_mosq, max_inflight_messages);
+}
+
 void mosquittopp::message_retry_set(unsigned int message_retry)
 {
 	mosquitto_message_retry_set(m_mosq, message_retry);
@@ -267,6 +292,11 @@ int mosquittopp::tls_opts_set(int cert_reqs, const char *tls_version, const char
 	return mosquitto_tls_opts_set(m_mosq, cert_reqs, tls_version, ciphers);
 }
 
+int mosquittopp::tls_insecure_set(bool value)
+{
+	return mosquitto_tls_insecure_set(m_mosq, value);
+}
+
 int mosquittopp::tls_psk_set(const char *psk, const char *identity, const char *ciphers)
 {
 	return mosquitto_tls_psk_set(m_mosq, psk, identity, ciphers);
