@@ -27,16 +27,17 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <string.h>
 
-#include <memory_mosq.h>
+#include "memory_mosq.h"
 
 #ifdef REAL_WITH_MEMORY_TRACKING
 #  if defined(__APPLE__)
-#    define malloc_usable_size malloc_good_size
+#    include <malloc/malloc.h>
+#    define malloc_usable_size malloc_size
 #  elif defined(__FreeBSD__)
 #    include <malloc_np.h>
 #  else
@@ -44,8 +45,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #  endif
 #endif
 
-#include <memory_mosq.h>
-
 #ifdef REAL_WITH_MEMORY_TRACKING
 static unsigned long memcount = 0;
 static unsigned long max_memcount = 0;
