@@ -31,11 +31,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <stdlib.h>
 #include <string.h>
 
-#include <mosquitto_internal.h>
-#include <mosquitto.h>
-#include <memory_mosq.h>
-#include <messages_mosq.h>
-#include <send_mosq.h>
+#include "mosquitto_internal.h"
+#include "mosquitto.h"
+#include "memory_mosq.h"
+#include "messages_mosq.h"
+#include "send_mosq.h"
+#include "time_mosq.h"
 
 void _mosquitto_message_cleanup(struct mosquitto_message_all **message)
 {
@@ -115,22 +116,22 @@ void mosquitto_message_free(struct mosquitto_message **message)
 
 void _mosquitto_message_queue(struct mosquitto *mosq, struct mosquitto_message_all *message)
 {
-	struct mosquitto_message_all *tail;
-
 	assert(mosq);
 	assert(message);
 
+	pthread_mutex_lock(&mosq->message_mutex);
 	mosq->queue_len++;
+	if(message->msg.qos > 0 && (mosq->max_inflight_messages == 0 || mosq->inflight_messages < mosq->max_inflight_messages)){
+		mosq->inflight_messages++;
+	}
+	pthread_mutex_unlock(&mosq->message_mutex);
 	message->next = NULL;
-	if(mosq->messages){
-		tail = mosq->messages;
-		while(tail->next){
-			tail = tail->next;
-		}
-		tail->next = message;
+	if(mosq->messages_last){
+		mosq->messages_last->next = message;
 	}else{
 		mosq->messages = message;
 	}
+	mosq->messages_last = message;
 }
 
 void _mosquitto_messages_reconnect_reset(struct mosquitto *mosq)
@@ -139,39 +140,58 @@ void _mosquitto_messages_reconnect_reset(struct mosquitto *mosq)
 	struct mosquitto_message_all *prev = NULL;
 	assert(mosq);
 
+	pthread_mutex_lock(&mosq->message_mutex);
 	mosq->queue_len = 0;
+	mosq->inflight_messages = 0;
 	message = mosq->messages;
 	while(message){
 		message->timestamp = 0;
 		if(message->direction == mosq_md_out){
-			if(message->msg.qos == 1){
-				message->state = mosq_ms_wait_puback;
-			}else if(message->msg.qos == 2){
-				message->state = mosq_ms_wait_pubrec;
-			}
 			mosq->queue_len++;
-		}else{
-			if(prev){
-				prev->next = message->next;
-				_mosquitto_message_cleanup(&message);
-				message = prev;
+			if(message->msg.qos > 0){
+				mosq->inflight_messages++;
+			}
+			if(mosq->max_inflight_messages == 0 || mosq->inflight_messages < mosq->max_inflight_messages){
+				if(message->msg.qos == 1){
+					message->state = mosq_ms_wait_for_puback;
+				}else if(message->msg.qos == 2){
+					/* Should be able to preserve state. */
+				}
 			}else{
-				mosq->messages = message->next;
-				_mosquitto_message_cleanup(&message);
-				message = mosq->messages;
+				message->state = mosq_ms_invalid;
+			}
+		}else{
+			if(message->msg.qos != 2){
+				if(prev){
+					prev->next = message->next;
+					_mosquitto_message_cleanup(&message);
+					message = prev;
+				}else{
+					mosq->messages = message->next;
+					_mosquitto_message_cleanup(&message);
+					message = mosq->messages;
+				}
+			}else{
+				/* Message state can be preserved here because it should match
+				 * whatever the client has got. */
 			}
 		}
 		prev = message;
 		message = message->next;
 	}
+	mosq->messages_last = prev;
+	pthread_mutex_unlock(&mosq->message_mutex);
 }
 
 int _mosquitto_message_remove(struct mosquitto *mosq, uint16_t mid, enum mosquitto_msg_direction dir, struct mosquitto_message_all **message)
 {
 	struct mosquitto_message_all *cur, *prev = NULL;
+	bool found = false;
+	int rc;
 	assert(mosq);
 	assert(message);
 
+	pthread_mutex_lock(&mosq->message_mutex);
 	cur = mosq->messages;
 	while(cur){
 		if(cur->msg.mid == mid && cur->direction == dir){
@@ -182,36 +202,74 @@ int _mosquitto_message_remove(struct mosquitto *mosq, uint16_t mid, enum mosquit
 			}
 			*message = cur;
 			mosq->queue_len--;
-			return MOSQ_ERR_SUCCESS;
+			if(!mosq->messages){
+				mosq->messages_last = NULL;
+			}
+			if((cur->msg.qos == 2 && dir == mosq_md_in) || (cur->msg.qos > 0 && dir == mosq_md_out)){
+				mosq->inflight_messages--;
+			}
+			found = true;
+			break;
 		}
 		prev = cur;
 		cur = cur->next;
 	}
-	return MOSQ_ERR_NOT_FOUND;
+
+	if(found){
+		cur = mosq->messages;
+		while(cur){
+			if(mosq->max_inflight_messages == 0 || mosq->inflight_messages < mosq->max_inflight_messages){
+				if(cur->msg.qos > 0 && cur->state == mosq_ms_invalid && cur->direction == mosq_md_out){
+					mosq->inflight_messages++;
+					if(cur->msg.qos == 1){
+						cur->state = mosq_ms_wait_for_puback;
+					}else if(cur->msg.qos == 2){
+						cur->state = mosq_ms_wait_for_pubrec;
+					}
+					rc = _mosquitto_send_publish(mosq, cur->msg.mid, cur->msg.topic, cur->msg.payloadlen, cur->msg.payload, cur->msg.qos, cur->msg.retain, cur->dup);
+					if(rc){
+						pthread_mutex_unlock(&mosq->message_mutex);
+						return rc;
+					}
+				}
+			}else{
+				pthread_mutex_unlock(&mosq->message_mutex);
+				return MOSQ_ERR_SUCCESS;
+			}
+			prev = cur;
+			cur = cur->next;
+		}
+		pthread_mutex_unlock(&mosq->message_mutex);
+		return MOSQ_ERR_SUCCESS;
+	}else{
+		pthread_mutex_unlock(&mosq->message_mutex);
+		return MOSQ_ERR_NOT_FOUND;
+	}
 }
 
 void _mosquitto_message_retry_check(struct mosquitto *mosq)
 {
 	struct mosquitto_message_all *message;
-	time_t now = time(NULL);
+	time_t now = mosquitto_time();
 	assert(mosq);
 
+	pthread_mutex_lock(&mosq->message_mutex);
 	message = mosq->messages;
 	while(message){
 		if(message->timestamp + mosq->message_retry < now){
 			switch(message->state){
-				case mosq_ms_wait_puback:
-				case mosq_ms_wait_pubrec:
+				case mosq_ms_wait_for_puback:
+				case mosq_ms_wait_for_pubrec:
 					message->timestamp = now;
 					message->dup = true;
 					_mosquitto_send_publish(mosq, message->msg.mid, message->msg.topic, message->msg.payloadlen, message->msg.payload, message->msg.qos, message->msg.retain, message->dup);
 					break;
-				case mosq_ms_wait_pubrel:
+				case mosq_ms_wait_for_pubrel:
 					message->timestamp = now;
 					message->dup = true;
 					_mosquitto_send_pubrec(mosq, message->msg.mid);
 					break;
-				case mosq_ms_wait_pubcomp:
+				case mosq_ms_wait_for_pubcomp:
 					message->timestamp = now;
 					message->dup = true;
 					_mosquitto_send_pubrel(mosq, message->msg.mid, true);
@@ -222,6 +280,7 @@ void _mosquitto_message_retry_check(struct mosquitto *mosq)
 		}
 		message = message->next;
 	}
+	pthread_mutex_unlock(&mosq->message_mutex);
 }
 
 void mosquitto_message_retry_set(struct mosquitto *mosq, unsigned int message_retry)
@@ -235,15 +294,27 @@ int _mosquitto_message_update(struct mosquitto *mosq, uint16_t mid, enum mosquit
 	struct mosquitto_message_all *message;
 	assert(mosq);
 
+	pthread_mutex_lock(&mosq->message_mutex);
 	message = mosq->messages;
 	while(message){
 		if(message->msg.mid == mid && message->direction == dir){
 			message->state = state;
-			message->timestamp = time(NULL);
+			message->timestamp = mosquitto_time();
+			pthread_mutex_unlock(&mosq->message_mutex);
 			return MOSQ_ERR_SUCCESS;
 		}
 		message = message->next;
 	}
+	pthread_mutex_unlock(&mosq->message_mutex);
 	return MOSQ_ERR_NOT_FOUND;
 }
 
+int mosquitto_max_inflight_messages_set(struct mosquitto *mosq, unsigned int max_inflight_messages)
+{
+	if(!mosq) return MOSQ_ERR_INVAL;
+
+	mosq->max_inflight_messages = max_inflight_messages;
+
+	return MOSQ_ERR_SUCCESS;
+}
+
