@@ -29,8 +29,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef _MESSAGES_MOSQ_H_
 #define _MESSAGES_MOSQ_H_
 
-#include <mosquitto_internal.h>
-#include <mosquitto.h>
+#include "mosquitto_internal.h"
+#include "mosquitto.h"
 
 void _mosquitto_message_cleanup_all(struct mosquitto *mosq);
 void _mosquitto_message_cleanup(struct mosquitto_message_all **message);
