@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2010,2011 Roger Light <roger@atchoo.org>
+Copyright (c) 2010,2011,2013 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef _MOSQUITTO_INTERNAL_H_
 #define _MOSQUITTO_INTERNAL_H_
 
-#include <config.h>
+#include "config.h"
 
 #ifdef WIN32
 #  include <winsock2.h>
@@ -40,7 +40,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <openssl/ssl.h>
 #endif
 #include <stdlib.h>
-#include <time.h>
 
 #if defined(WITH_THREADING) && !defined(WITH_BROKER)
 #  include <pthread.h>
@@ -61,7 +60,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #	include <stdint.h>
 #endif
 
-#include <mosquitto.h>
+#include "mosquitto.h"
+#include "time_mosq.h"
 #ifdef WITH_BROKER
 struct mosquitto_client_msg;
 #endif
@@ -73,17 +73,25 @@ enum mosquitto_msg_direction {
 
 enum mosquitto_msg_state {
 	mosq_ms_invalid = 0,
-	mosq_ms_wait_puback = 1,
-	mosq_ms_wait_pubrec = 2,
-	mosq_ms_wait_pubrel = 3,
-	mosq_ms_wait_pubcomp = 4
+	mosq_ms_publish_qos0 = 1,
+	mosq_ms_publish_qos1 = 2,
+	mosq_ms_wait_for_puback = 3,
+	mosq_ms_publish_qos2 = 4,
+	mosq_ms_wait_for_pubrec = 5,
+	mosq_ms_resend_pubrel = 6,
+	mosq_ms_wait_for_pubrel = 7,
+	mosq_ms_resend_pubcomp = 8,
+	mosq_ms_wait_for_pubcomp = 9,
+	mosq_ms_send_pubrec = 10,
+	mosq_ms_queued = 11
 };
 
 enum mosquitto_client_state {
 	mosq_cs_new = 0,
 	mosq_cs_connected = 1,
 	mosq_cs_disconnecting = 2,
-	mosq_cs_connect_async = 3
+	mosq_cs_connect_async = 3,
+	mosq_cs_connect_pending = 4
 };
 
 struct _mosquitto_packet{
@@ -143,6 +151,7 @@ struct mosquitto {
 	char *tls_ciphers;
 	char *tls_psk;
 	char *tls_psk_identity;
+	bool tls_insecure;
 #endif
 	bool want_read;
 	bool want_write;
@@ -153,6 +162,7 @@ struct mosquitto {
 	pthread_mutex_t out_packet_mutex;
 	pthread_mutex_t current_out_packet_mutex;
 	pthread_mutex_t state_mutex;
+	pthread_mutex_t message_mutex;
 	pthread_t thread_id;
 #endif
 #ifdef WITH_BROKER
@@ -163,6 +173,8 @@ struct mosquitto {
 	struct _mqtt3_listener *listener;
 	time_t disconnect_t;
 	int pollfd_index;
+	int db_index;
+	struct _mosquitto_packet *out_packet_last;
 #else
 	void *userdata;
 	bool in_callback;
@@ -180,6 +192,15 @@ struct mosquitto {
 	char *host;
 	int port;
 	int queue_len;
+	char *bind_address;
+	unsigned int reconnect_delay;
+	unsigned int reconnect_delay_max;
+	bool reconnect_exponential_backoff;
+	bool threaded;
+	struct _mosquitto_packet *out_packet_last;
+	struct mosquitto_message_all *messages_last;
+	int inflight_messages;
+	int max_inflight_messages;
 #endif
 };
 
