@@ -44,14 +44,18 @@ def on_subscribe(mosq, obj, mid, granted_qos):
 def on_log(mosq, obj, level, string):
     print(string)
 
-mqttc = mosquitto.Mosquitto("python_sub")
+# If you want to use a specific client id, use
+# mqttc = mosquitto.Mosquitto("client-id")
+# but note that the client id must be unique on the broker. Leaving the client
+# id parameter empty will generate a random id for you.
+mqttc = mosquitto.Mosquitto()
 mqttc.on_message = on_message
 mqttc.on_connect = on_connect
 mqttc.on_publish = on_publish
 mqttc.on_subscribe = on_subscribe
 # Uncomment to enable debug messages
 #mqttc.on_log = on_log
-mqttc.connect("127.0.0.1", 1883, 60)
+mqttc.connect("test.mosquitto.org", 1883, 60)
 mqttc.subscribe("$SYS/#", 0)
 
 
