@@ -31,15 +31,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <stdio.h>
 #include <string.h>
 
-#include <mosquitto.h>
-#include <logging_mosq.h>
-#include <memory_mosq.h>
-#include <messages_mosq.h>
-#include <mqtt3_protocol.h>
-#include <net_mosq.h>
-#include <read_handle.h>
-#include <send_mosq.h>
-#include <util_mosq.h>
+#include "mosquitto.h"
+#include "logging_mosq.h"
+#include "memory_mosq.h"
+#include "messages_mosq.h"
+#include "mqtt3_protocol.h"
+#include "net_mosq.h"
+#include "read_handle.h"
+#include "send_mosq.h"
+#include "time_mosq.h"
+#include "util_mosq.h"
 
 int _mosquitto_packet_handle(struct mosquitto *mosq)
 {
@@ -131,7 +132,7 @@ int _mosquitto_handle_publish(struct mosquitto *mosq)
 			message->msg.mid, message->msg.topic,
 			(long)message->msg.payloadlen);
 
-	message->timestamp = time(NULL);
+	message->timestamp = mosquitto_time();
 	switch(message->msg.qos){
 		case 0:
 			pthread_mutex_lock(&mosq->callback_mutex);
@@ -156,7 +157,7 @@ int _mosquitto_handle_publish(struct mosquitto *mosq)
 			return rc;
 		case 2:
 			rc = _mosquitto_send_pubrec(mosq, message->msg.mid);
-			message->state = mosq_ms_wait_pubrel;
+			message->state = mosq_ms_wait_for_pubrel;
 			_mosquitto_message_queue(mosq, message);
 			return rc;
 		default:
