@@ -44,7 +44,7 @@ int mqtt3_handle_connack(struct mosquitto_db *db, struct mosquitto *context)
 	int i;
 	char *notification_topic;
 	int notification_topic_len;
-	char notification_payload[2];
+	char notification_payload;
 
 	if(!context){
 		return MOSQ_ERR_INVAL;
@@ -61,30 +61,28 @@ int mqtt3_handle_connack(struct mosquitto_db *db, struct mosquitto *context)
 		case CONNACK_ACCEPTED:
 			if(context->bridge){
 				if(context->bridge->notifications){
-					notification_payload[0] = '1';
-					notification_payload[1] = '\0';
+					notification_payload = '1';
 					if(context->bridge->notification_topic){
 						if(_mosquitto_send_real_publish(context, _mosquitto_mid_generate(context),
-								context->bridge->notification_topic, 2, &notification_payload, 1, true, 0)){
+								context->bridge->notification_topic, 1, &notification_payload, 1, true, 0)){
 
 							return 1;
 						}
-						mqtt3_db_messages_easy_queue(db, context, context->bridge->notification_topic, 1, 2, &notification_payload, 1);
+						mqtt3_db_messages_easy_queue(db, context, context->bridge->notification_topic, 1, 1, &notification_payload, 1);
 					}else{
 						notification_topic_len = strlen(context->id)+strlen("$SYS/broker/connection//state");
 						notification_topic = _mosquitto_malloc(sizeof(char)*(notification_topic_len+1));
 						if(!notification_topic) return MOSQ_ERR_NOMEM;
 
 						snprintf(notification_topic, notification_topic_len+1, "$SYS/broker/connection/%s/state", context->id);
-						notification_payload[0] = '1';
-						notification_payload[1] = '\0';
+						notification_payload = '1';
 						if(_mosquitto_send_real_publish(context, _mosquitto_mid_generate(context),
-								notification_topic, 2, &notification_payload, 1, true, 0)){
+								notification_topic, 1, &notification_payload, 1, true, 0)){
 
 							_mosquitto_free(notification_topic);
 							return 1;
 						}
-						mqtt3_db_messages_easy_queue(db, context, notification_topic, 1, 2, &notification_payload, 1);
+						mqtt3_db_messages_easy_queue(db, context, notification_topic, 1, 1, &notification_payload, 1);
 						_mosquitto_free(notification_topic);
 					}
 				}
