@@ -36,22 +36,26 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <mqtt3_protocol.h>
 #include <memory_mosq.h>
 #include <send_mosq.h>
+#include <time_mosq.h>
+#include <tls_mosq.h>
 #include <util_mosq.h>
 
+#ifdef WITH_SYS_TREE
 extern unsigned int g_connection_count;
+#endif
 
 int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 {
-	char *protocol_name;
+	char *protocol_name = NULL;
 	uint8_t protocol_version;
 	uint8_t connect_flags;
-	char *client_id;
+	char *client_id = NULL;
 	char *will_payload = NULL, *will_topic = NULL;
 	uint16_t will_payloadlen;
 	struct mosquitto_message *will_struct = NULL;
 	uint8_t will, will_retain, will_qos, clean_session;
 	uint8_t username_flag, password_flag;
-	char *username, *password = NULL;
+	char *username = NULL, *password = NULL;
 	int i;
 	int rc;
 	struct _mosquitto_acl_user *acl_tail;
@@ -61,8 +65,12 @@ int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 	X509_NAME *name;
 	X509_NAME_ENTRY *name_entry;
 #endif
+	struct _clientid_index_hash *find_cih;
+	struct _clientid_index_hash *new_cih;
 
+#ifdef WITH_SYS_TREE
 	g_connection_count++;
+#endif
 
 	/* Don't accept multiple CONNECT commands. */
 	if(context->state != mosq_cs_new){
@@ -133,6 +141,7 @@ int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 #else
 	if(slen == 0){
 #endif
+		_mosquitto_free(client_id);
 		_mosquitto_send_connack(context, CONNACK_REFUSED_IDENTIFIER_REJECTED);
 		mqtt3_context_disconnect(db, context);
 		return 1;
@@ -151,33 +160,39 @@ int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 	if(will){
 		will_struct = _mosquitto_calloc(1, sizeof(struct mosquitto_message));
 		if(!will_struct){
-			_mosquitto_free(client_id);
 			mqtt3_context_disconnect(db, context);
-			return MOSQ_ERR_NOMEM;
+			rc = MOSQ_ERR_NOMEM;
+			goto handle_connect_error;
 		}
 		if(_mosquitto_read_string(&context->in_packet, &will_topic)){
-			_mosquitto_free(client_id);
 			mqtt3_context_disconnect(db, context);
-			return 1;
+			rc = 1;
+			goto handle_connect_error;
+		}
+		if(strlen(will_topic) == 0){
+			/* FIXME - CONNACK_REFUSED_IDENTIFIER_REJECTED not really appropriate here. */
+			_mosquitto_send_connack(context, CONNACK_REFUSED_IDENTIFIER_REJECTED);
+			mqtt3_context_disconnect(db, context);
+			rc = 1;
+			goto handle_connect_error;
 		}
 		if(_mosquitto_read_uint16(&context->in_packet, &will_payloadlen)){
-			_mosquitto_free(client_id);
 			mqtt3_context_disconnect(db, context);
-			return 1;
+			rc = 1;
+			goto handle_connect_error;
 		}
 		will_payload = _mosquitto_malloc(will_payloadlen);
 		if(!will_payload){
-			_mosquitto_free(client_id);
 			mqtt3_context_disconnect(db, context);
-			return 1;
+			rc = 1;
+			goto handle_connect_error;
 		}
 
 		rc = _mosquitto_read_bytes(&context->in_packet, will_payload, will_payloadlen);
 		if(rc){
-			_mosquitto_free(will_payload);
-			_mosquitto_free(client_id);
 			mqtt3_context_disconnect(db, context);
-			return 1;
+			rc = 1;
+			goto handle_connect_error;
 		}
 	}
 
@@ -187,17 +202,16 @@ int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 			if(password_flag){
 				rc = _mosquitto_read_string(&context->in_packet, &password);
 				if(rc == MOSQ_ERR_NOMEM){
-					_mosquitto_free(username);
-					_mosquitto_free(client_id);
-					return MOSQ_ERR_NOMEM;
+					rc = MOSQ_ERR_NOMEM;
+					goto handle_connect_error;
 				}else if(rc == MOSQ_ERR_PROTOCOL){
 					/* Password flag given, but no password. Ignore. */
 					password_flag = 0;
 				}
 			}
 		}else if(rc == MOSQ_ERR_NOMEM){
-			_mosquitto_free(client_id);
-			return MOSQ_ERR_NOMEM;
+			rc = MOSQ_ERR_NOMEM;
+			goto handle_connect_error;
 		}else{
 			/* Username flag given, but no username. Ignore. */
 			username_flag = 0;
@@ -209,143 +223,142 @@ int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 		if(!context->ssl){
 			_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
 			mqtt3_context_disconnect(db, context);
-			_mosquitto_free(client_id);
-			return MOSQ_ERR_SUCCESS;
+			rc = MOSQ_ERR_SUCCESS;
+			goto handle_connect_error;
 		}
-#ifdef WITH_TLS_PSK
+#ifdef REAL_WITH_TLS_PSK
 		if(context->listener->psk_hint){
 			/* Client should have provided an identity to get this far. */
 			if(!context->username){
 				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
 				mqtt3_context_disconnect(db, context);
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_SUCCESS;
+				rc = MOSQ_ERR_SUCCESS;
+				goto handle_connect_error;
 			}
 		}else{
-#endif /* WITH_TLS_PSK */
+#endif /* REAL_WITH_TLS_PSK */
 			client_cert = SSL_get_peer_certificate(context->ssl);
 			if(!client_cert){
 				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
 				mqtt3_context_disconnect(db, context);
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_SUCCESS;
+				rc = MOSQ_ERR_SUCCESS;
+				goto handle_connect_error;
 			}
 			name = X509_get_subject_name(client_cert);
 			if(!name){
 				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
 				mqtt3_context_disconnect(db, context);
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_SUCCESS;
+				rc = MOSQ_ERR_SUCCESS;
+				goto handle_connect_error;
 			}
 
 			i = X509_NAME_get_index_by_NID(name, NID_commonName, -1);
 			if(i == -1){
 				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
 				mqtt3_context_disconnect(db, context);
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_SUCCESS;
+				rc = MOSQ_ERR_SUCCESS;
+				goto handle_connect_error;
 			}
 			name_entry = X509_NAME_get_entry(name, i);
 			context->username = _mosquitto_strdup((char *)ASN1_STRING_data(name_entry->value));
 			if(!context->username){
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_NOMEM;
+				rc = MOSQ_ERR_SUCCESS;
+				goto handle_connect_error;
 			}
-#ifdef WITH_TLS_PSK
+#ifdef REAL_WITH_TLS_PSK
 		}
-#endif /* WITH_TLS_PSK */
+#endif /* REAL_WITH_TLS_PSK */
 	}else{
 #endif /* WITH_TLS */
 		if(username_flag){
 			rc = mosquitto_unpwd_check(db, username, password);
-			context->username = username;
-			context->password = password;
 			if(rc == MOSQ_ERR_AUTH){
 				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
 				mqtt3_context_disconnect(db, context);
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_SUCCESS;
+				rc = MOSQ_ERR_SUCCESS;
+				goto handle_connect_error;
 			}else if(rc == MOSQ_ERR_INVAL){
-				_mosquitto_free(client_id);
-				return MOSQ_ERR_INVAL;
+				goto handle_connect_error;
 			}
+			context->username = username;
+			context->password = password;
+			username = NULL; /* Avoid free() in error: below. */
+			password = NULL;
 		}
 
 		if(!username_flag && db->config->allow_anonymous == false){
 			_mosquitto_send_connack(context, CONNACK_REFUSED_NOT_AUTHORIZED);
 			mqtt3_context_disconnect(db, context);
-			_mosquitto_free(client_id);
-			return MOSQ_ERR_SUCCESS;
+			rc = MOSQ_ERR_SUCCESS;
+			goto handle_connect_error;
 		}
 #ifdef WITH_TLS
 	}
 #endif
 
 	/* Find if this client already has an entry. This must be done *after* any security checks. */
-	for(i=0; i<db->context_count; i++){
-		if(db->contexts[i] && db->contexts[i]->id && !strcmp(db->contexts[i]->id, client_id)){
-			/* Client does match. */
-			if(db->contexts[i]->sock == -1){
-				/* Client is reconnecting after a disconnect */
-				/* FIXME - does anything else need to be done here? */
-			}else{
-				/* Client is already connected, disconnect old version */
-				if(db->config->connection_messages == true){
-					_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Client %s already connected, closing old connection.", client_id);
-				}
+	HASH_FIND_STR(db->clientid_index_hash, client_id, find_cih);
+	if(find_cih){
+		i = find_cih->db_context_index;
+		/* Found a matching client */
+		if(db->contexts[i]->sock == -1){
+			/* Client is reconnecting after a disconnect */
+			/* FIXME - does anything else need to be done here? */
+		}else{
+			/* Client is already connected, disconnect old version */
+			if(db->config->connection_messages == true){
+				_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Client %s already connected, closing old connection.", client_id);
 			}
-			db->contexts[i]->clean_session = clean_session;
-			mqtt3_context_cleanup(db, db->contexts[i], false);
-			db->contexts[i]->state = mosq_cs_connected;
-			db->contexts[i]->address = _mosquitto_strdup(context->address);
-			db->contexts[i]->sock = context->sock;
-			db->contexts[i]->listener = context->listener;
-			db->contexts[i]->last_msg_in = time(NULL);
-			db->contexts[i]->last_msg_out = time(NULL);
-			db->contexts[i]->keepalive = context->keepalive;
-			db->contexts[i]->pollfd_index = context->pollfd_index;
+		}
+		db->contexts[i]->clean_session = clean_session;
+		mqtt3_context_cleanup(db, db->contexts[i], false);
+		db->contexts[i]->state = mosq_cs_connected;
+		db->contexts[i]->address = _mosquitto_strdup(context->address);
+		db->contexts[i]->sock = context->sock;
+		db->contexts[i]->listener = context->listener;
+		db->contexts[i]->last_msg_in = mosquitto_time();
+		db->contexts[i]->last_msg_out = mosquitto_time();
+		db->contexts[i]->keepalive = context->keepalive;
+		db->contexts[i]->pollfd_index = context->pollfd_index;
 #ifdef WITH_TLS
-			db->contexts[i]->ssl = context->ssl;
+		db->contexts[i]->ssl = context->ssl;
 #endif
-			if(context->username){
-				db->contexts[i]->username = _mosquitto_strdup(context->username);
-			}
-			context->sock = -1;
+		if(context->username){
+			db->contexts[i]->username = _mosquitto_strdup(context->username);
+		}
+		context->sock = -1;
 #ifdef WITH_TLS
-			context->ssl = NULL;
+		context->ssl = NULL;
 #endif
-			context->state = mosq_cs_disconnecting;
-			context = db->contexts[i];
-			if(context->msgs){
-				mqtt3_db_message_reconnect_reset(context);
-			}
-			break;
+		context->state = mosq_cs_disconnecting;
+		context = db->contexts[i];
+		if(context->msgs){
+			mqtt3_db_message_reconnect_reset(context);
 		}
 	}
 
 	context->id = client_id;
+	client_id = NULL;
 	context->clean_session = clean_session;
 	context->ping_t = 0;
 
+	// Add the client ID to the DB hash table here
+	new_cih = _mosquitto_malloc(sizeof(struct _clientid_index_hash));
+	if(!new_cih){
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
+		mqtt3_context_disconnect(db, context);
+		rc = MOSQ_ERR_NOMEM;
+		goto handle_connect_error;
+	}
+	new_cih->id = context->id;
+	new_cih->db_context_index = context->db_index;
+	HASH_ADD_KEYPTR(hh, db->clientid_index_hash, context->id, strlen(context->id), new_cih);
+
 #ifdef WITH_PERSISTENCE
 	if(!clean_session){
 		db->persistence_changes++;
 	}
 #endif
-	context->will = will_struct;
-	if(context->will){
-		context->will->topic = will_topic;
-		if(will_payload){
-			context->will->payload = will_payload;
-			context->will->payloadlen = will_payloadlen;
-		}else{
-			context->will->payload = NULL;
-			context->will->payloadlen = 0;
-		}
-		context->will->qos = will_qos;
-		context->will->retain = will_retain;
-	}
-
 	/* Associate user with its ACL, assuming we have ACLs loaded. */
 	if(db->acl_list){
 		acl_tail = db->acl_list;
@@ -367,12 +380,40 @@ int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 		context->acl_list = NULL;
 	}
 
+	if(will_struct){
+		if(mosquitto_acl_check(db, context, will_topic, MOSQ_ACL_WRITE) != MOSQ_ERR_SUCCESS){
+			_mosquitto_send_connack(context, CONNACK_REFUSED_NOT_AUTHORIZED);
+			mqtt3_context_disconnect(db, context);
+			rc = MOSQ_ERR_SUCCESS;
+			goto handle_connect_error;
+		}
+		context->will = will_struct;
+		context->will->topic = will_topic;
+		if(will_payload){
+			context->will->payload = will_payload;
+			context->will->payloadlen = will_payloadlen;
+		}else{
+			context->will->payload = NULL;
+			context->will->payloadlen = 0;
+		}
+		context->will->qos = will_qos;
+		context->will->retain = will_retain;
+	}
+
 	if(db->config->connection_messages == true){
-		_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "New client connected from %s as %s.", context->address, client_id);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "New client connected from %s as %s (c%d, k%d).", context->address, context->id, context->clean_session, context->keepalive);
 	}
 
 	context->state = mosq_cs_connected;
 	return _mosquitto_send_connack(context, CONNACK_ACCEPTED);
+
+handle_connect_error:
+	if(client_id) _mosquitto_free(client_id);
+	if(username) _mosquitto_free(username);
+	if(will_payload) _mosquitto_free(will_payload);
+	if(will_topic) _mosquitto_free(will_topic);
+	if(will_struct) _mosquitto_free(will_struct);
+	return rc;
 }
 
 int mqtt3_handle_disconnect(struct mosquitto_db *db, struct mosquitto *context)
@@ -461,6 +502,7 @@ int mqtt3_handle_subscribe(struct mosquitto_db *db, struct mosquitto *context)
 			}else if(rc2 != -1){
 				rc = rc2;
 			}
+			_mosquitto_log_printf(NULL, MOSQ_LOG_SUBSCRIBE, "%s %d %s", context->id, qos, sub);
 			_mosquitto_free(sub);
 
 			tmp_payload = _mosquitto_realloc(payload, payloadlen + 1);
@@ -505,6 +547,7 @@ int mqtt3_handle_unsubscribe(struct mosquitto_db *db, struct mosquitto *context)
 		if(sub){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_DEBUG, "\t%s", sub);
 			mqtt3_sub_remove(db, context, sub, &db->subs);
+			_mosquitto_log_printf(NULL, MOSQ_LOG_UNSUBSCRIBE, "%s %s", context->id, sub);
 			_mosquitto_free(sub);
 		}
 	}
