@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 # Test whether a client can connect without an SSL certificate if one is required.
 # 
@@ -31,13 +31,16 @@ broker = subprocess.Popen(['../../src/mosquitto', '-c', '08-ssl-connect-cert-aut
 time.sleep(0.5)
 
 sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-ssock = ssl.wrap_socket(sock, ca_certs="../ssl/test-ca.crt", cert_reqs=ssl.CERT_REQUIRED)
-ssock.settimeout(5)
+ssock = ssl.wrap_socket(sock, ca_certs="../ssl/test-root-ca.crt", cert_reqs=ssl.CERT_REQUIRED)
+ssock.settimeout(20)
 try:
     ssock.connect(("localhost", 1888))
 except ssl.SSLError as err:
     if err.errno == 1:
         rc = 0
+except socket.error as err:
+    if err.errno == 104:
+        rc = 0
 
 broker.terminate()
 broker.wait()
